@@ -203,6 +203,9 @@ class JSON(Renderer):
     def serialized_columns(self):
         return {col.key: str(col.label) for col in self.columns}
 
+    def serialized_column_types(self):
+        return {col.key: col.json_type_helper for col in self.columns}
+
     def serialize_column_group(self, label, columns):
         return types.ColumnGroup(
             label=label,
@@ -235,14 +238,6 @@ class JSON(Renderer):
             if col.filter.is_active
         }
 
-    def serialize_filter_operator(self, op):
-        return types.FilterOperator(
-            key=op.key,
-            label=op.display,
-            field_type=op.field_type,
-            hint=op.hint,
-        )
-
     def serialized_filter_specs(self):
         return {
             col.key: col.filter.serialize_filter_spec()
@@ -262,6 +257,9 @@ class JSON(Renderer):
     def serialized_order_by(self):
         return [self.serialize_sort(sort) for sort in self.grid.order_by]
 
+    def serialized_sortable_columns(self):
+        return [col.key for col in filter(lambda col: col.can_sort, self.grid.columns)]
+
     def asdict(self):
         grid = types.Grid(
             errors=[],
@@ -279,9 +277,11 @@ class JSON(Renderer):
             spec=types.GridSpec(
                 columns=self.serialized_columns(),
                 column_groups=self.serialized_column_groups(),
+                column_types=self.serialized_column_types(),
                 export_targets=list(self.grid.allowed_export_targets.keys()),
                 enable_search=self.grid.enable_search,
                 enable_sort=self.grid.sorter_on,
+                sortable_columns=self.serialized_sortable_columns(),
                 filters=self.serialized_filter_specs(),
             ),
             state=types.GridState(
